import typer
from wssiham.tools.configuration import Configuration
from wssiham.tools.typerExtend import TyperExtend
from wssiham.tools.toolbox import Toolbox
from rich.progress import Progress, SpinnerColumn, TextColumn, TimeElapsedColumn
from rich.console import Console
from rich.table import Table
from rich.prompt import IntPrompt

import pathlib
import json
import logging

logger = logging.getLogger(__name__)
app = typer.Typer()


@app.command()
def create(certificat: bool = typer.Option(False, "--cert", "-c", help="Possède un certificat crt"),
           default_wsdl: bool = typer.Option(False, "--wsdl", "-w", help="Générer les chemins wsdl par défaut")):
    """ Création d'un fichier de configuration du webservice SIHAM
    """

    filename = typer.prompt("Nom de votre fichier de configuration")
    user = typer.prompt("Nom d'utilisateur du webservice")
    password = typer.prompt("Mot de passe du webservice", confirmation_prompt=True, hide_input=True)
    base_wsdl = typer.prompt("Url du webservice (exemple : https://monsiham.univ.fr:8081/) ")
    if certificat:
        cert = typer.prompt("Chemin du certificat (crt) ")
    else:
        cert = ""

    Configuration.createws(f'{pathlib.Path(__file__).parents[1]}/config/{filename}', filename, user, password,
                           base_wsdl, cert, default_wsdl)

    TyperExtend.valid_echo(f'Création du fichier du fichier de configuation {filename}.ini')


@app.command()
def listing():
    """ Liste les fichiers de configuration existants
    """

    directory = pathlib.Path(f'{pathlib.Path(__file__).parents[1]}/config/')
    files = list(directory.glob('*.ini'))
    TyperExtend.simple_echo('Liste des fichiers de configurations disponible dans le répertoire de config')
    TyperExtend.breakline()
    table = Table(title=f'Nombre de configuration trouvé : {len(files)}')
    table.add_column("Chemin du fichier", justify="left", style="cyan", no_wrap=True)
    table.add_column("Fichier", justify="left")
    table.add_column("Dernière modification", justify="left", style="green")
    table.add_column("Configuration par défaut", justify="center", style="red")
    for file in files:
        envname = Toolbox.environnement_name()
        flag = ''
        if envname == pathlib.Path(file).stem:
            flag = '*'
        table.add_row(str(Toolbox.abs_path(file).parent),
                      pathlib.Path(file).name,
                      Toolbox.get_modification_date_file(file),
                      flag)
    console = Console()
    console.print(table)


@app.command()
def cat(file: str = typer.Option("", "--file", "-f", help="fichier de configuration")):
    """ Affiche le fichier de configuration par défaut
    """
    console = Console()
    if file:
        if '.ini' not in file or pathlib.Path(file).exists():
            TyperExtend.error_echo('Le fichier de configuration n\'existe pas ou ne possède pas de .ini')
            raise TyperExtend.exit_unsuccess()
        else:
            confile = f"{pathlib.Path(__file__).parents[1]}/config/{file}"
        # directory = pathlib.Path(f'{pathlib.Path(__file__).parents[1]}/config/')
        # files = list(directory.glob('*.ini'))
        # for cfile in files:
        #     if pathlib.Path(cfile).name == file:
        #         TyperExtend.simple_echo(f'Affichage de la configuration du fichier {file}')
        #         TyperExtend.breakline()
            console.print_json(json.dumps(Configuration.loadws(confile)), indent=4)
    else:
        TyperExtend.simple_echo('Affichage de la configuration du fichier par défaut pour l\'accès au webservice SIHAM')
        TyperExtend.breakline()
        console.print_json(json.dumps(Configuration.loadws()), indent=4)


@app.command()
def load(file: str = typer.Option("", "--file", "-f", help="nom du fichier de configuration webservice SIHAM à charger")):
    """ Charge le fichier de configuration
    """

    if file:
        if '.ini' not in file or pathlib.Path(file).exists():
            TyperExtend.error_echo('Le fichier de configuration n\'existe pas ou ne possède pas de .ini')
            raise TyperExtend.exit_unsuccess()
        else:
            confile = f"{pathlib.Path(__file__).parents[1]}/config/{file}"
    else:
        TyperExtend.simple_echo('Liste des fichiers de configuration disponible dans le dossier config')
        TyperExtend.breakline()

        directory = pathlib.Path(f'{pathlib.Path(__file__).parents[1]}/config/')
        inifiles = sorted(list(directory.glob('*.ini')))

        # Créer une liste des positions des fichiers dans la liste triée
        positions = [str(inifiles.index(file) + 1) for file in inifiles]

        for position, inifile in zip(positions, inifiles):
            TyperExtend.simple_echo(f'{position}) {inifile.name}')
        TyperExtend.breakline()
        numcomf = IntPrompt.ask("Choississez la configuration à charger", choices=positions)

        confile = f"{pathlib.Path(__file__).parents[1]}/config/{inifiles[numcomf - 1].name}"
    if confile:
        data = Configuration.loadws(confile)
        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            TimeElapsedColumn(),
            transient=False,
        ) as progress:
            progress.add_task(description="Initialisation des paramètres ...",
                              total=None)
            Configuration.create_confmod(data['ENVIRONMENT_NAME'], f'config/{pathlib.Path(confile).name}')

        TyperExtend.valid_echo('Chargemenet réussi !')
    else:
        TyperExtend.error_echo('Une erreur est survenu')
        raise TyperExtend.exit_unsuccess()


@app.command()
def remove(file: str = typer.Option("", "--file", "-f", help="nom du fichier de configuration à supprimer")):
    """ Supprime un fichier de configuration
    """

    if file:
        if '.ini' not in file or pathlib.Path(file).exists():
            TyperExtend.error_echo('Le fichier de configuration n\'existe pas ou ne possède pas de .ini')
            raise TyperExtend.exit_unsuccess()
        else:
            confile = f"{pathlib.Path(__file__).parents[1]}/config/{file}"
    else:
        TyperExtend.simple_echo('Liste des fichiers de configuration disponible dans le dossier config')
        TyperExtend.breakline()

        directory = pathlib.Path(f'{pathlib.Path(__file__).parents[1]}/config/')
        inifiles = sorted(list(directory.glob('*.ini')))

        # Créer une liste des positions des fichiers dans la liste triée
        positions = [str(inifiles.index(file) + 1) for file in inifiles]

        for position, inifile in zip(positions, inifiles):
            TyperExtend.simple_echo(f'{position}) {inifile.name}')
        TyperExtend.breakline()
        numcomf = IntPrompt.ask("Choississez la configuration à supprimer", choices=positions)

        confile = f"{pathlib.Path(__file__).parents[1]}/config/{inifiles[numcomf - 1].name}"
    if confile:

        pathlib.Path(confile).unlink()
        TyperExtend.valid_echo('Suppression de la configuration réussi !')
    else:
        TyperExtend.error_echo('Une erreur est survenu')
        raise TyperExtend.exit_unsuccess()
