from dataclasses import dataclass, field
from wssiham.app.pec_doctorant.classes.doctorant import Doctorant
from wssiham.app.pec_doctorant.classes.doctorantSiham import DoctorantSiham
from typing import Dict, List
from pathlib import Path
import pandas as pd
import logging
import sys

logger = logging.getLogger(__name__)


@dataclass
class Gateway:
    """
    Classe faisant le lien entre les webservices RechercheAgent et PriseEnChargeAgent pour l'injection des doctorants.
    """
    path: str                                           # Chemin du fichier excel contenant les doctorants
    output_directory: str                               # Répertoire qui contiendra les fichiers rapports
    start_date_employment: str                          # dd/mm/yyyy
    end_date_employment: str                            # dd/mm/yyyy
    data: List[Dict] = field(init=False, repr=False)    # Donnée correspondant au fichier excel sous forme de json

    def __post_init__(self):
        if Path(self.path).exists() or Path(self.output_directory).is_dir():
            df = pd.read_excel(self.path)
            self.data = df.to_dict(orient='records')
        else:
            logger.debug(f'Le fichier {self.path} ou le dossier n\'existe pas ')
            sys.exit(1)

    def process(self, simulation: bool = True) -> None:
        """ Fonction principale pour l'injection des doctorants dans SIHAM.
        Si une correspondance (RechercheAgent) existe pour un doctorant, alors on ne fait rien.
        Si aucune correspondance n'était trouvée, alors on injecte les informations du doctarant dans SIHAM.
        """
        def rapport_csv(rprt: List[str]) -> str:
            """ Crée un msg csv contenant les informations suivantes pour une liste donnée :
            - Nom
            - Prénom
            - Date de naissance
            - Matricule Siham
            - Crappy URL

            Args:
                rprt (List[str]): Une liste contenant les informations suivantes
                ['nom', 'prenom', 'date_naissance', 'matricule_siham', 'crappy_url]

            Returns:
                Une chaine de caractère formaté en csv avec comme séparateur ','

            Examples:
                >>> ra = rapport_csv(['John', 'Doe', 'RUN000011223', 'https://crappy_url.re'])
                >>> print(ra)
                NOM,PRENOM,DATE_NAISSANCE,MATRICULE_SIHAM,CRAPPY_URL\n
                John,Doe,RUN000011223,https://crappy_url.re
            """
            data = 'NOM,PRENOM,DATE_NAISSANCE,MATRICULE_SIHAM,CRAPPY_URL\n'
            data += ''.join(rprt)

            return data

        def injection_csv(injec: List[str]) -> str:
            """ Crée un msg csv contenant les informations d'injection. Ces informations proviennent des données
            utilisées par le webservice PriseEnChargeAgent

            Args:
                injec (List[str]): Représentation, sous format csv, des données qui ont été injectés dans SIHAM

            Returns:
                Représentation sous forme de csv des données qui ont été injectés dans SIHAM
            """
            res = 'NOM,PRENOM,CODE_CIVILITE,CODE_SEXE,CATEGORIE_ENTREE,CODE_ADMINISTRATION,CODE_ETABLISSEMENT,'\
              'DATE_EMBAUCHE,DATE_NAISSANCE,CODE_UO,CODE_EMPLOI,DATE_EFFET_STATUT,CODE_STATUT,DATE_EFFET_POSITION,'\
              'DATE_FIN_REELLE_POSITION,DATE_FIN_PREVUE_POSITION,CODE_POSITION,' \
              'DATE_EFFET_MODALITE,CODE_MODALITE,TEMOIN_VALIDITE,STATUT_INJECTION\n'

            res += ''.join(injec)

            return res

        def save(msg_rapport: str, msg_injection: str) -> None:
            """ Crée 2 fichiers, rappport.csv et injection.csv, contenants les données.

            Args:
                msg_rapport (str): Donnée formatée en csv pour le fichier rapport.csv
                msg_injection (str): Donnée formatée en csv pour le fichier injection.csv

            Returns:

            """
            with open(r := f'{self.output_directory}/rapport.csv', 'w') as fdr:
                logger.info(f'Création du fichier {r}')
                logger.debug(msg_rapport)
                fdr.write(msg_rapport)
                logger.info(f'Sauvegarde du fichier {r}')
            with open(i := f'{self.output_directory}/injection.csv', 'w') as fdi:
                logger.info(f'Création du fichier {i}')
                logger.debug(msg_injection)
                fdi.write(msg_injection)
                logger.debug(f'Sauvegarde du fichier {i}')

        tmp_rapport = []
        tmp_injection = []

        for ind in self.data:
            genre = ind["Civilité"]
            nom = ind["Nom"].strip()
            prenom = ind["Prenom"].strip()
            # numero_etudiant = str(ind["Numéro étudiant"])
            date_naissance = ind["Date de naissance"].strip()
            unite_de_recherche = ind["Unité de recherche"].strip()

            if ind.get("Numéro étudiant"):
                numero_etudiant = str(ind["Numéro étudiant"])

            student = Doctorant(genre, nom, prenom, date_naissance, unite_de_recherche)
            # student = Doctorant(genre, nom, prenom, date_naissance, unite_de_recherche, numero_etudiant)

            is_sihamed = student.est_dans_siham()

            # Si une correspondance a été trouvée
            if is_sihamed:
                logger.info('Une correspondance a été trouvée avec le webservice RechercheAgent')
                logger.info('Ajout des données formatées en csv dans une liste temporaire')
                tmp_rapport.append(student.csvinfo())

            # Sinon, on injecte le doctorant dans Siham
            else:
                logger.info('Aucune correspondance n\'a été trouvée avec le webservice RechercheAgent')
                del student
                doctorant = DoctorantSiham(nom=nom, prenom=prenom, genre=genre, date_naissance=date_naissance,
                                           unite_recherche=unite_de_recherche,
                                           # numero_etudiant=numero_etudiant,
                                           date_embauche=self.start_date_employment,
                                           date_effet_statut=self.start_date_employment,
                                           date_effet_position=self.start_date_employment,
                                           date_fin_position=self.end_date_employment,
                                           date_effet_modalite=self.start_date_employment)

                if not simulation:
                    ############################
                    # Injection des doctorannts
                    logger.debug(f'Injection du doctorant {nom} {prenom}')
                    doctorant.injection()
                    ############################

                logger.info('Ajout des données formatées en csv dans les listes temporaires')
                tmp_injection.append(doctorant.data_to_csv())
                tmp_rapport.append(doctorant.csvinfo())

        rapport = rapport_csv(tmp_rapport)
        injection = injection_csv(tmp_injection)
        save(rapport, injection)

